#!/usr/bin/env python
# -*- coding: utf-8 -*-

from bout_install.main import bout_install_command_line

# NOTE: No __name__ == '__main__' to avoid double import trap
# https://stackoverflow.com/questions/43393764/python-3-6-project-structure-leads-to-runtimewarning/45070583#45070583
bout_install_command_line()



from bout_install.Installer import Installer


class InstallerUsingCMake(Installer):
    """
    An Installer class which installs configures using `cmake` rather than 
    `./configure`
    """

    @staticmethod
    def get_cmake_command(cmake_options=None):
        """
        Get the command to cmake the package

        Notes
        -----
        The `..` at the end of cmake is automatically appended

        Parameters
        ----------
        cmake_options : dict
            Configuration options to use with `cmake`.
            The configuration options will be converted to `--key=val` during
            runtime

        Returns
        -------
        cmake_str : str
            The configuration command
        """

        options = ''
        if cmake_options is not None:
            for key, val in cmake_options.items():
                options += f' -{key}={val}'

        cmake_str = f'cmake{options} ..'
        return cmake_str

    def run_cmake(self,
                  build_dir,
                  makefile_path,
                  extra_cmake_option,
                  overwrite_on_exist):
        """
        Configures the package by running CMake

        Parameters
        ----------
        build_dir : Path
            Directory to make the build
        makefile_path : Path
            Path to the Makefile
        extra_cmake_option:
            Configure option to include.
            -DCMAKE_INSTALL_PREFIX=self.local_dir is already added as an option
        overwrite_on_exist : bool
            Whether to overwrite the package if it is already found
        """

        if not makefile_path.is_file() or overwrite_on_exist:
            # Get the CMake options
            cmake_options = dict(DCMAKE_INSTALL_PREFIX=str(self.local_dir))
            if extra_cmake_option is not None:
                cmake_options = {**cmake_options, **extra_cmake_option}

            cmake_str = self.get_cmake_command(cmake_options=cmake_options)

            # NOTE: We could get in trouble if the build_dir already exists,
            #       so we deliberately throw an error if the directory exists
            build_dir.mkdir(exist_ok=False)

            self.logger.info(f'Running cmake with: {cmake_str}')
            self.run_subprocess(cmake_str, build_dir)
        else:
            self.logger.info(f'{makefile_path} found, skipping running of '
                             f'CMake')

    def install_package(self,
                        url,
                        file_from_make,
                        path_config_log='',
                        overwrite_on_exist=False,
                        extra_cmake_option=None):
        """
        Installs a package if it's not installed

        Parameters
        ----------
        url : str
            Url to the tar file of the package
        file_from_make : Path or str
            File originating from the make processes (used to check if the
            package has been made)
        path_config_log : str or Path
            Only used in parent class
        overwrite_on_exist : bool
            Whether to overwrite the package if it is already found
        extra_cmake_option : dict
            Configure option to include.
            The installation prefix of self.local_dir is already added as an
            option
        """

        # Download the tar file
        tar_file_path = self.get_tar_file_path(url)
        self.run_download_tar(url, tar_file_path, overwrite_on_exist)

        # Untar
        tar_dir = self.get_tar_dir(tar_file_path)
        self.run_untar(tar_file_path, tar_dir, overwrite_on_exist)

        # Configure and make
        build_dir = tar_dir.joinpath('build')
        makefile_path = build_dir.joinpath('Makefile')
        self.run_cmake(build_dir,
                       makefile_path,
                       extra_cmake_option,
                       overwrite_on_exist)
        self.run_make(build_dir, file_from_make, overwrite_on_exist)



#!/usr/bin/env python
# -*- coding: utf-8 -*-


import shutil
import configparser
from pathlib import Path


class BaseTestSetup(object):
    """
    A class for facilitating the set up and tear down of the tests
    """

    def __init__(self, test_name):
        """
        Sets the paths and the path name

        Parameters
        ----------
        test_name : str
            The name of the test (in order to not get concurrency issues as
            tests are run in parallel)
        """
        root_dir = Path(__file__).absolute().parents[1]

        self.main_dir = root_dir.joinpath(f'test_main_dir_{test_name}')
        self.other_dir = root_dir.joinpath(f'test_other_dir_{test_name}')
        self.config_ini_path = root_dir.joinpath('bout_install', 'config.ini')
        self.test_config_ini_path = \
            self.config_ini_path.parent.joinpath(f'config_{test_name}.ini')

        # Initialize
        self.config = None

    def set_up(self):
        """
        Copy and modify the original config.ini
        """

        # Copy the content of config.ini
        shutil.copy(self.config_ini_path, self.test_config_ini_path)

        # Replace the main_dir
        self.config = configparser.ConfigParser(allow_no_value=True)
        with self.test_config_ini_path.open() as f:
            self.config.read_file(f)

        self.config['install_options']['main_dir'] = str(self.main_dir)
        self.config['bout_options']['git_dir'] = \
            str(self.main_dir.joinpath('BOUT-dev'))
        with self.test_config_ini_path.open('w') as f:
            self.config.write(f)

    def tear_down(self):
        """
        Remove created directories and files, restore config.ini
        """

        shutil.rmtree(self.main_dir, ignore_errors=True)
        shutil.rmtree(self.other_dir, ignore_errors=True)
        self.test_config_ini_path.unlink()



import shutil
from pathlib import Path
from bout_install.Installer import Installer


class HDF5Installer(Installer):
    """
    Installer object for installing HDF5
    """

    def __init__(self,
                 config_path=Path(__file__).parent.joinpath('config.ini'),
                 log_path=Path(__file__).parents[1].joinpath('log', 'hdf5.log'),
                 overwrite_on_exist=False):
        """
        Gets the HDF5 version, sets the HDF5 url and calls the super constructor

        Parameters
        ----------
        config_path : Path or str
            The path to the get_configure_command file
        log_path : None or Path or str
            Path to the log file containing the log of Installer.
            If None, the log will directed to stderr
        overwrite_on_exist : bool
            Whether to overwrite the package if it is already found
        """

        self.overwrite_on_exist = overwrite_on_exist

        super().__init__(config_path=config_path, log_path=log_path)

        self.hdf5_version = self.config['versions']['hdf5']
        hdf5_major_minor_version = '.'.join(self.hdf5_version.split('.')[:2])
        self.hdf5_url = (f'https://support.hdfgroup.org/ftp/HDF5/releases/'
                         f'hdf5-{hdf5_major_minor_version}/'
                         f'hdf5-{self.hdf5_version}/src/'
                         f'hdf5-{self.hdf5_version}.tar.gz')
        self.file_from_make = self.local_dir.joinpath('bin', 'h5c++')

        self.extra_config_options = {'enable-cxx': 'yes'}

    def install(self):
        """
        Installs the HDF5 package
        """

        self.logger.info('Installing HDF5')

        if shutil.which('h5c++') is None or not self.use_preinstalled:
            self.install_package(url=self.hdf5_url,
                                 file_from_make=self.file_from_make,
                                 extra_config_option=self.extra_config_options,
                                 overwrite_on_exist=self.overwrite_on_exist)
            self.logger.info('Installation completed successfully')
        else:
            self.logger.info('Found h5c++ in PATH, skipping...')



import shutil
from pathlib import Path
from bout_install.Installer import Installer


class InstallerUsingGit(Installer):
    """
    An Installer class which installs using git rather than tarballs
    """

    def __init__(self,
                 name,
                 section,
                 config_path=Path(__file__).parent.joinpath('config.ini'),
                 log_path=None):
        """
        Makes the logger and installation paths (obtained from config.ini)

        Notes
        -----
        `self.local_dir.joinpath('bin')` will be set to the environments
        `PATH` variable and `self.local_dir.joinpath('lib')` will be set to the
        environments `LD_LIBRARY_PATH` variable in order to ensure a proper
        installation

        Parameters
        ----------
        section : str
            What section to use in the config file
        config_path : Path or str
            The path to the get_configure_command file
        log_path : None or Path or str
            Path to the log file containing the log of Installer.
            If None, the log will directed to stderr
        """

        super().__init__(config_path=config_path, log_path=log_path)

        # Obtain install dirs
        git_dir = self.config[section]['git_dir']
        checkout = self.config[section]['checkout']

        if git_dir == '':
            self.git_dir = Path().home().joinpath(name)
        else:
            self.git_dir = Path(git_dir)

        self.checkout = checkout if checkout != '' else 'master'

    def make(self, path):
        """
        Make the package

        Notes
        -----
        Will not make install

        Parameters
        ----------
        path : Path or str
            Path to the get_configure_command file
        """

        make_str = 'make'
        self.run_subprocess(make_str, path)

    def run_git(self, url, overwrite_on_exist=False):
        """
        Runs git

        Parameters
        ----------
        url : str
            URL to the package repository
        overwrite_on_exist : bool
            Whether to overwrite the package if it is already found
        """

        if not self.git_dir.is_dir() or overwrite_on_exist:
            if self.git_dir.is_dir():
                shutil.rmtree(str(self.git_dir))
            command = f'git clone {url} {self.git_dir}'
            self.run_subprocess(command, self.git_dir.parent)

    def install_package(self,
                        url,
                        file_from_make,
                        path_config_log='config.log',
                        overwrite_on_exist=False,
                        extra_config_option=None):
        """
        Installs a package if it's not installed

        Parameters
        ----------
        url : str
            Url to the package repository
        file_from_make : Path or str
            File originating from the make processes (used to check if the
            package has been made)
        path_config_log : str or Path
            Name of the log file for configure relative to the configuration
            file
        overwrite_on_exist : bool
            Whether to overwrite the package if it is already found
        extra_config_option : dict
            Configure option to include.
            The installation prefix of self.local_dir is already added as an
            option
        """

        # Run git checkout
        self.run_git(url, overwrite_on_exist)

        # Configure and make
        config_log_path = self.git_dir.joinpath(path_config_log)
        self.run_configure(self.git_dir,
                           config_log_path,
                           extra_config_option,
                           overwrite_on_exist)
        self.run_make(self.git_dir, file_from_make, overwrite_on_exist)



from pathlib import Path
from bout_install.Installer import Installer


class GCCInstaller(Installer):
    """
    Installer object for installing GCC
    """

    def __init__(self,
                 config_path=Path(__file__).parent.joinpath('config.ini'),
                 log_path=Path(__file__).parents[1].joinpath('log', 'gcc.log'),
                 overwrite_on_exist=False):
        """
        Gets the GCC version, sets the GCC url and calls the super constructor

        Parameters
        ----------
        config_path : Path or str
            The path to the get_configure_command file
        log_path : None or Path or str
            Path to the log file containing the log of Installer.
            If None, the log will directed to stderr
        overwrite_on_exist : bool
            Whether to overwrite the package if it is already found
        """

        self.overwrite_on_exist = overwrite_on_exist

        super().__init__(config_path=config_path, log_path=log_path)

        self.gcc_version = self.config['versions']['gcc']
        self.gcc_url = (f'http://mirror.koddos.net/gcc/releases/'
                        f'gcc-{self.gcc_version}/gcc-{self.gcc_version}.tar.gz')
        self.file_from_make = self.local_dir.joinpath('bin', 'gcc')

    def install_package(self,
                        url,
                        file_from_make,
                        path_config_log='config.log',
                        overwrite_on_exist=False,
                        extra_config_option=None):
        """
        Installs GCC if it's not installed

        Parameters
        ----------
        url : str
            Url to the tar file of the package
        file_from_make : Path or str
            File originating from the make processes (used to check if the
            package has been made)
        path_config_log : str or Path
            Name of the log file for configure relative to the configuration
            file
        overwrite_on_exist : bool
            Whether to overwrite the package if it is already found
        extra_config_option : dict
            Configure option to include.
            --prefix=self.local_dir is already added as an option

        References
        ----------
        http://luiarthur.github.io/gccinstall
        """

        # Download the tar file
        tar_file_path = self.get_tar_file_path(url)
        self.run_download_tar(url, tar_file_path, overwrite_on_exist)

        # Untar
        tar_dir = self.get_tar_dir(tar_file_path)
        self.run_untar(tar_file_path, tar_dir, overwrite_on_exist)

        # Download prerequisites
        self.logger.info('Downloading prerequisites')
        prereq_path = Path('contrib').joinpath('download_prerequisites')
        self.run_subprocess(f'./{prereq_path}', tar_dir)

        # Configure and make
        config_log_path = tar_dir.joinpath(path_config_log)
        self.run_configure(tar_dir,
                           config_log_path,
                           extra_config_option,
                           overwrite_on_exist)
        self.run_make(tar_dir, file_from_make, overwrite_on_exist)

    def install(self):
        """
        Installs the GCC package
        """

        self.logger.info('Installing GCC')
        self.install_package(url=self.gcc_url,
                             file_from_make=self.file_from_make,
                             overwrite_on_exist=self.overwrite_on_exist)
        self.logger.info('Installation completed successfully')



from pathlib import Path
from bout_install.Installer import Installer
from bout_install.installer.PETScInstaller import PETScInstaller


class SLEPcInstaller(Installer):
    """
    Installer object for installing SLEPc
    """

    def __init__(self,
                 config_path=Path(__file__).parent.joinpath('config.ini'),
                 slepc_log_path=Path(__file__).parents[1].joinpath('log',
                                                                   'slepc.log'),
                 petsc_log_path=Path(__file__).parents[1].joinpath('log',
                                                                   'petsc.log'),
                 mpi_log_path=Path(__file__).parents[1].joinpath('log',
                                                                 'mpi.log'),
                 overwrite_on_exist=False):
        """
        Gets the SLEPc version, sets the SLEPc url and calls the super
        constructor

        Parameters
        ----------
        config_path : Path or str
            The path to the get_configure_command file
        slepc_log_path : None or Path or str
            Path to the log file for SLEPc
            If None, the log will directed to stderr
        petsc_log_path : None or Path or str
            Path to the log file for PETSc
            If None, the log will directed to stderr
        mpi_log_path : None or Path or str
            Path to the log file for MPI
            If None, the log will directed to stderr
        overwrite_on_exist : bool
            Whether to overwrite the package if it is already found
        """

        self.overwrite_on_exist = overwrite_on_exist

        super().__init__(config_path=config_path, log_path=slepc_log_path)

        self.petsc_version = self.config['versions']['petsc']
        self.slepc_version = self.config['versions']['slepc']

        # Create dependency installer
        self.petsc = PETScInstaller(config_path=config_path,
                                    petsc_log_path=petsc_log_path,
                                    mpi_log_path=mpi_log_path)

        self.slepc_url = (f'http://slepc.upv.es/download/distrib/'
                          f'slepc-{self.slepc_version}.tar.gz')

        self.file_from_make = self.local_dir.joinpath('lib', 'libslepc.a')

    @staticmethod
    def get_configure_command(config_options=None):
        """
        Get the command to configure the package.

        Notes
        -----
        Configuring happens through python 2

        Parameters
        ----------
        config_options : dict
            Configuration options to use with `./configure`.
            The configuration options will be converted to `--key=val` during
            runtime

        Returns
        -------
        config_str : str
            The configuration command
        """

        options = ''
        if config_options is not None:
            for key, val in config_options.items():
                if val is not None:
                    options += f' --{key}={val}'
                else:
                    options += f' --{key}'

        config_str = f'python2 ./configure{options}'
        return config_str

    def make(self, path):
        """
        Make the package using make all and make test

        Parameters
        ----------
        path : Path or str
            Path to the get_configure_command file
        """

        make_options = \
            (f'SLEPC_DIR={self.install_dir}/slepc-{self.slepc_version}'
             f' PETSC_DIR={self.local_dir}')

        make_str = f'make {make_options}'
        self.run_subprocess(make_str, path)

        make_install_str = f'make {make_options} install'
        self.run_subprocess(make_install_str, path)

        make_test_options =  \
            (f'SLEPC_DIR={self.local_dir}'
             f' PETSC_DIR={self.local_dir}'
             f' PETSC_ARCH=')
        make_test_str = f'make {make_test_options} check'
        self.run_subprocess(make_test_str, path)

    def install(self):
        """
        Installs the SLEPc package and its dependencies
        """

        self.install_dependencies()

        # Set config log path
        # NOTE: The configuration log is hiding in strange places in SLEPc,
        #       we'll therefore try to glob us to the configure.log
        tar_dir = Path(self.get_tar_file_path(self.slepc_url)).\
            absolute().parent.joinpath(f'slepc-{self.slepc_version}')
        path_config_logs = sorted(tar_dir.glob('**/configure.log'))
        if len(path_config_logs) == 0:
            # No configuration file found, might as well set it to configure.log
            path_config_log = 'configure.log'
        else:
            # Configuration files found, use the first
            path_config_log = path_config_logs[0]

        self.logger.info('Installing SLEPc')
        self.install_package(url=self.slepc_url,
                             file_from_make=self.file_from_make,
                             path_config_log=path_config_log,
                             overwrite_on_exist=self.overwrite_on_exist)
        self.logger.info('Installation completed successfully')

    def install_dependencies(self):
        """
        Installs SLEPc dependencies
        """

        self.petsc.install()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import re
import unittest
import shutil
import subprocess
from pathlib import Path
from bout_install.main import install_bout
from bout_install.main import add_str_to_bashrc
from tests.utils import BaseTestSetup


class TestMain(unittest.TestCase):
    def setUp(self):
        """
        Set up global test parameters, and modify config.ini

        A back-up of config.ini is made prior to modification
        """

        self.base_setup = BaseTestSetup('boutpp')
        self.base_setup.set_up()

        # Setup the config path
        self.config = self.base_setup.test_config_ini_path

    def tearDown(self):
        """
        Remove created directories and files, restore config.ini
        """

        self.base_setup.tear_down()

    def test_main(self):
        """
        Test the installation and that we can run blob2d with PETSc
        """

        install_bout(self.config)

        # We now try to make blob 2d
        # The os.environ from BOUTPPInstaller should persist, so no need of
        # running it again

        # Copy to data/
        blob2d_dir = self.base_setup.main_dir.joinpath('BOUT-dev',
                                                       'examples',
                                                       'blob2d')
        data_dir = blob2d_dir.joinpath('data')
        delta_025_dir = blob2d_dir.joinpath('delta_0.25')
        shutil.copytree(delta_025_dir, data_dir)

        # Change BOUT.inp lines
        n_out = re.compile(r'^\bNOUT\b', re.IGNORECASE)
        time_step = re.compile(r'^\bTIMESTEP\b', re.IGNORECASE)
        boussinesq = re.compile(r'^\bboussinesq\b', re.IGNORECASE)

        bout_inp_path = data_dir.joinpath('BOUT.inp')
        with bout_inp_path.open('r') as f:
            bout_inp_list = f.readlines()
            for i in range(len(bout_inp_list)):
                if n_out.search(bout_inp_list[i]):
                    bout_inp_list[i] = 'NOUT = 2\n'
                elif time_step.search(bout_inp_list[i]):
                    bout_inp_list[i] = 'TIMESTEP = 1e-5\n'
                elif boussinesq.search(bout_inp_list[i]):
                    bout_inp_list[i] = 'boussinesq = false\n'

        with bout_inp_path.open('w') as f:
            f.writelines(bout_inp_list)

        # Make
        command = 'make'
        result = subprocess.run(command.split(),
                                cwd=blob2d_dir,
                                stdout=subprocess.PIPE,
                                stderr=subprocess.PIPE)
        result.check_returncode()

        # Run
        command = 'mpirun -np 2 ./blob2d'
        result = subprocess.run(command.split(),
                                cwd=blob2d_dir,
                                stdout=subprocess.PIPE,
                                stderr=subprocess.PIPE)
        result.check_returncode()


class TestMainHelpers(unittest.TestCase):
    def setUp(self):
        """
        Make a backup of .bashrc (makes the file if it doesn't exist)
        """

        self.bashrc_path = Path.home().joinpath('.bashrc')
        self.exist = self.bashrc_path.is_file()
        if not self.exist:
            self.bashrc_path.touch()

        self.bashrc_bak_path = Path.home().joinpath('.bashrc.bak')
        shutil.copy(self.bashrc_path, self.bashrc_bak_path)

    def tearDown(self):
        """
        Move .bashrc.bak to .bashrc
        """

        shutil.copy(self.bashrc_bak_path, self.bashrc_path)
        self.bashrc_bak_path.unlink()

        if not self.exist:
            self.bashrc_path.unlink()

    def test_add_str_to_bashrc(self):
        """
        Test that we can add to bashrc
        """

        expected = '# harmless string'
        add_str_to_bashrc(expected)

        with self.bashrc_path.open('r') as f:
            lines = f.readlines()

        self.assertEqual(expected, lines[-1])


if __name__ == '__main__':
    unittest.main()




import shutil
from pathlib import Path
from bout_install.Installer import Installer


class FFMPEGInstaller(Installer):
    """
    Installer object for installing FFMPEG
    """

    def __init__(self,
                 config_path=Path(__file__).parent.joinpath('config.ini'),
                 ffmpeg_log_path=
                 Path(__file__).parents[1].joinpath('log', 'ffmpeg.log'),
                 nasm_log_path=
                 Path(__file__).parents[1].joinpath('log', 'nasm.log'),
                 yasm_log_path=
                 Path(__file__).parents[1].joinpath('log', 'yasm.log'),
                 x264_log_path=
                 Path(__file__).parents[1].joinpath('log', 'x264.log'),
                 overwrite_on_exist=False):
        """
        Gets the version and url of FFMPEG and calls the super constructor

        The constructor will also make objects of the dependency installers

        Parameters
        ----------
        config_path : Path or str
            The path to the get_configure_command file
        ffmpeg_log_path : None or Path or str
            Path to the log file for FFMPEG
            If None, the log will directed to stderr
        nasm_log_path : None or Path or str
            Path to the log file for NASM
            If None, the log will directed to stderr
        yasm_log_path : None or Path or str
            Path to the log file for YASM
            If None, the log will directed to stderr
        x264_log_path : None or Path or str
            Path to the log file for X264
            If None, the log will directed to stderr
        overwrite_on_exist : bool
            Whether to overwrite the package if it is already found
        """

        self.overwrite_on_exist = overwrite_on_exist

        super().__init__(config_path=config_path, log_path=ffmpeg_log_path)

        self.ffmpeg_version = self.config['versions']['ffmpeg']
        self.ffmpeg_url = (f'http://www.ffmpeg.org/releases/ffmpeg-'
                           f'{self.ffmpeg_version}.tar.gz')
        self.file_from_make = self.local_dir.joinpath('bin', 'ffmpeg')

        # Create dependency installers
        self.nasm = \
            NASMInstaller(config_path=config_path, log_path=nasm_log_path)
        self.yasm = \
            YASMInstaller(config_path=config_path, log_path=yasm_log_path)
        self.x264 = \
            X264Installer(config_path=config_path, log_path=x264_log_path)

        self.extra_config_options = \
            {'enable-gpl': None,
             'enable-libx264': None,
             'extra-ldflags': f'-L{self.local_dir.joinpath("lib")}',
             'extra-cflags': f'-I{self.local_dir.joinpath("include")}'}

    def install(self):
        """
        Installs the FFMPEG package and its dependencies
        """

        self.install_dependencies()

        self.logger.info('Installing FFMPEG')

        if shutil.which('ffmpeg') is None or not self.use_preinstalled:
            self.install_package(url=self.ffmpeg_url,
                                 file_from_make=self.file_from_make,
                                 extra_config_option=self.extra_config_options,
                                 overwrite_on_exist=self.overwrite_on_exist)
            self.logger.info('Installation completed successfully')
        else:
            self.logger.info('Found ffmpeg in PATH, skipping...')

    def install_dependencies(self):
        """
        Installs FFMPEG dependencies
        """

        self.nasm.install()
        self.yasm.install()
        self.x264.install()


class NASMInstaller(Installer):
    """
    Installer object for installing NASM
    """

    def __init__(self,
                 config_path=Path(__file__).parent.joinpath('config.ini'),
                 log_path=Path(__file__).parents[1].joinpath('log', 'nasm.log'),
                 overwrite_on_exist=False):
        """
        Gets the NASM version, sets the NASM url and calls the super
        constructor

        Parameters
        ----------
        config_path : Path or str
            The path to the get_configure_command file
        log_path : None or Path or str
            Path to the log file containing the log of Installer.
            If None, the log will directed to stderr
        overwrite_on_exist : bool
            Whether to overwrite the package if it is already found
        """

        self.overwrite_on_exist = overwrite_on_exist

        super().__init__(config_path=config_path, log_path=log_path)

        self.nasm_version = self.config['versions']['nasm']
        self.nasm_url = (f'http://www.nasm.us/pub/nasm/releasebuilds/'
                         f'{self.nasm_version}/'
                         f'nasm-{self.nasm_version}.tar.gz')

        self.file_from_make = self.local_dir.joinpath('bin', 'nasm')

    def install(self):
        """
        Installs the NASM package
        """

        self.logger.info('Installing NASM')

        if shutil.which('nasm') is None or not self.use_preinstalled:
            self.install_package(url=self.nasm_url,
                                 file_from_make=self.file_from_make,
                                 overwrite_on_exist=self.overwrite_on_exist)
            self.logger.info('Installation completed successfully')
        else:
            self.logger.info('Found nasm in PATH, skipping...')


class YASMInstaller(Installer):
    """
    Installer object for installing YASM
    """

    def __init__(self,
                 config_path=Path(__file__).parent.joinpath('config.ini'),
                 log_path=Path(__file__).parents[1].joinpath('log', 'yasm.log'),
                 overwrite_on_exist=False):
        """
        Gets the YASM version, sets the YASM url and calls the super
        constructor

        Parameters
        ----------
        config_path : Path or str
            The path to the get_configure_command file
        log_path : None or Path or str
            Path to the log file containing the log of Installer.
            If None, the log will directed to stderr
        overwrite_on_exist : bool
            Whether to overwrite the package if it is already found
        """

        self.overwrite_on_exist = overwrite_on_exist

        super().__init__(config_path=config_path, log_path=log_path)

        self.yasm_version = self.config['versions']['yasm']
        self.yasm_url = (
            f'http://www.tortall.net/projects/yasm/releases/yasm-'
            f'{self.yasm_version}.tar.gz')

        self.file_from_make = self.local_dir.joinpath('bin', 'yasm')

    def install(self):
        """
        Installs the YASM package
        """

        self.logger.info('Installing YASM')

        if shutil.which('yasm') is None or not self.use_preinstalled:
            self.install_package(url=self.yasm_url,
                                 file_from_make=self.file_from_make,
                                 overwrite_on_exist=self.overwrite_on_exist)
            self.logger.info('Installation completed successfully')
        else:
            self.logger.info('Found yasm in PATH, skipping...')


class X264Installer(Installer):
    """
    Installer object for installing X264
    """

    def __init__(self,
                 config_path=Path(__file__).parent.joinpath('config.ini'),
                 log_path=Path(__file__).parents[1].joinpath('log', 'x264.log'),
                 overwrite_on_exist=False):
        """
        Gets the X264 version, sets the X264 url and calls the super
        constructor

        Parameters
        ----------
        config_path : Path or str
            The path to the get_configure_command file
        log_path : None or Path or str
            Path to the log file containing the log of Installer.
            If None, the log will directed to stderr
        overwrite_on_exist : bool
            Whether to overwrite the package if it is already found
        """

        self.overwrite_on_exist = overwrite_on_exist

        super().__init__(config_path=config_path, log_path=log_path)

        self.x264_version = self.config['versions']['x264']
        self.x264_url = (f'https://download.videolan.org/pub/videolan/x264/' 
                         f'snapshots/{self.x264_version}.tar.bz2')

        self.file_from_make = self.local_dir.joinpath('bin', 'x264')

        self.extra_config_options = {'enable-static': None,
                                     'enable-shared': None}

    def install(self):
        """
        Installs the X264 package
        """

        self.logger.info('Installing X264')

        if shutil.which('x264') is None or not self.use_preinstalled:
            self.install_package(url=self.x264_url,
                                 file_from_make=self.file_from_make,
                                 extra_config_option=self.extra_config_options,
                                 overwrite_on_exist=self.overwrite_on_exist)
            self.logger.info('Installation completed successfully')
        else:
            self.logger.info('Found x264 in PATH, skipping...')



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import unittest
from bout_install.Installer import Installer
from tests.utils import BaseTestSetup


class TestInstall(unittest.TestCase):
    def setUp(self):
        """
        Set up global test parameters, and modify config.ini

        A back-up of config.ini is made prior to modification
        """

        self.base_setup = BaseTestSetup('install')
        self.base_setup.set_up()

        # Get the main dir and other dir
        self.main_dir = self.base_setup.main_dir
        self.other_dir = self.base_setup.other_dir

        # Setup the config path
        self.config = self.base_setup.test_config_ini_path

        # Get the fftw url
        fftw_version = self.base_setup.config['versions']['fftw']
        self.fftw_url = f'http://www.fftw.org/fftw-{fftw_version}.tar.gz'

        self.installer = Installer(config_path=self.config)

    def tearDown(self):
        """
        Remove created directories and files, restore config.ini
        """

        self.base_setup.tear_down()

    def test__setup_logger(self):
        """
        Test that the logger is working
        """

        log_path = self.main_dir.joinpath('test.log')
        installer = Installer(config_path=self.config,
                              log_path=log_path)
        installer.logger.info('This is a test')
        self.assertTrue(log_path.is_file())

    def test_set_install_dirs(self):
        """
        Tests that the directories are properly installed
        """

        self.installer.setup_install_dirs(main_dir=self.main_dir)
        install_dir = self.main_dir.joinpath('install')
        local_dir = self.main_dir.joinpath('local')
        examples_dir = self.main_dir.joinpath('examples')
        self.assertTrue(install_dir.is_dir())
        self.assertTrue(local_dir.is_dir())
        self.assertTrue(examples_dir.is_dir())

        install_dir = self.other_dir.joinpath('install')
        local_dir = self.other_dir.joinpath('local')
        examples_dir = self.other_dir.joinpath('examples')
        self.installer.setup_install_dirs(install_dir=install_dir,
                                          local_dir=local_dir,
                                          examples_dir=examples_dir)
        self.assertTrue(install_dir.is_dir())
        self.assertTrue(local_dir.is_dir())
        self.assertTrue(examples_dir.is_dir())

    def test_get_tar_file(self):
        """
        Tests that the .tar files can be downloaded
        """

        self.installer.setup_install_dirs(main_dir=self.main_dir)
        tar_file_path = \
            self.installer.get_tar_file_path(url=self.fftw_url)

        self.installer.get_tar_file(url=self.fftw_url)
        self.assertTrue(tar_file_path.is_file())

    def test_untar(self):
        """
        Tests for successful untaring
        """

        self.installer.setup_install_dirs(main_dir=self.main_dir)
        tar_file_path = \
            self.installer.get_tar_file_path(url=self.fftw_url)

        self.installer.get_tar_file(url=self.fftw_url)
        self.installer.untar(tar_file_path)

        tar_dir = self.installer.get_tar_dir(tar_file_path)
        self.assertTrue(tar_dir.is_dir())

    def test_configure(self):
        """
        Test for successful configuring
        """

        self.installer.setup_install_dirs(main_dir=self.main_dir)
        tar_file_path = \
            self.installer.get_tar_file_path(url=self.fftw_url)
        self.installer.get_tar_file(url=self.fftw_url)

        self.installer.untar(tar_file_path)

        config_options = dict(prefix=str(self.installer.local_dir))
        config_str = \
            self.installer.get_configure_command(config_options=config_options)

        tar_dir = self.installer.get_tar_dir(tar_file_path)
        self.installer.run_subprocess(config_str, tar_dir)
        self.assertTrue(tar_dir.joinpath('config.log').is_file())

    def test_make(self):
        """
        Test for successful making
        """

        self.installer.setup_install_dirs(main_dir=self.main_dir)
        tar_file_path = \
            self.installer.get_tar_file_path(url=self.fftw_url)

        self.installer.get_tar_file(url=self.fftw_url)
        self.installer.untar(tar_file_path)

        config_options = dict(prefix=str(self.installer.local_dir))
        config_str = \
            self.installer.get_configure_command(config_options=config_options)

        tar_dir = self.installer.get_tar_dir(tar_file_path)
        self.installer.run_subprocess(config_str, tar_dir)
        self.installer.make(tar_dir)
        bin_file = self.installer.local_dir.joinpath('bin', 'fftw-wisdom')
        self.assertTrue(bin_file.is_file())

    def test_install_package(self):
        """
        Tests that the install wrapper is working
        """

        self.installer.setup_install_dirs(main_dir=self.main_dir)
        bin_file = self.installer.local_dir.joinpath('bin', 'fftw-wisdom')
        self.installer.install_package(self.fftw_url, bin_file)
        self.assertTrue(bin_file.is_file())


if __name__ == '__main__':
    unittest.main()




import shutil
from pathlib import Path
from bout_install.Installer import Installer


class MPIInstaller(Installer):
    """
    Installer object for installing MPI
    """

    def __init__(self,
                 config_path=Path(__file__).parent.joinpath('config.ini'),
                 log_path=Path(__file__).parents[1].joinpath('log', 'mpi.log'),
                 overwrite_on_exist=False):
        """
        Gets the MPI version, sets the MPI url and calls the super constructor

        Parameters
        ----------
        config_path : Path or str
            The path to the get_configure_command file
        log_path : None or Path or str
            Path to the log file containing the log of Installer.
            If None, the log will directed to stderr
        overwrite_on_exist : bool
            Whether to overwrite the package if it is already found
        """

        self.overwrite_on_exist = overwrite_on_exist

        super().__init__(config_path=config_path, log_path=log_path)

        self.mpi_version = self.config['versions']['mpi']
        self.mpi_url = (f'http://www.mpich.org/static/downloads/'
                        f'{self.mpi_version}/mpich-{self.mpi_version}.tar.gz')
        self.file_from_make = self.local_dir.joinpath('bin', 'mpicxx')

    def install(self):
        """
        Installs the MPI package
        """

        self.logger.info('Installing MPI')

        if shutil.which('mpicxx') is None or not self.use_preinstalled:
            self.install_package(url=self.mpi_url,
                                 file_from_make=self.file_from_make,
                                 overwrite_on_exist=self.overwrite_on_exist)
            self.logger.info('Installation completed successfully')
        else:
            self.logger.info('mpicxx found in PATH, skipping...')



import shutil
from pathlib import Path
from bout_install.Installer import Installer
from bout_install.installer.MPIInstaller import MPIInstaller


class PETScInstaller(Installer):
    """
    Installer object for installing PETSc
    """

    def __init__(self,
                 config_path=Path(__file__).parent.joinpath('config.ini'),
                 petsc_log_path=Path(__file__).parents[1].joinpath('log',
                                                                   'petsc.log'),
                 mpi_log_path=Path(__file__).parents[1].joinpath('log',
                                                                 'mpi.log'),
                 overwrite_on_exist=False):
        """
        Gets the version and url of PETSc and calls the super constructor

        The constructor will also make an object of the MPI installer and

        Parameters
        ----------
        config_path : Path or str
            The path to the get_configure_command file
        petsc_log_path : None or Path or str
            Path to the log file for PETSc
            If None, the log will directed to stderr
        mpi_log_path : None or Path or str
            Path to the log file for MPI
            If None, the log will directed to stderr
        overwrite_on_exist : bool
            Whether to overwrite the package if it is already found
        """

        self.overwrite_on_exist = overwrite_on_exist

        super().__init__(config_path=config_path, log_path=petsc_log_path)

        self.petsc_version = self.config['versions']['petsc']
        self.petsc_url = (f'http://ftp.mcs.anl.gov/pub/petsc/release-snapshots/'
                          f'petsc-{self.petsc_version}.tar.gz')

        # Create dependency installer
        self.mpi = MPIInstaller(config_path=config_path,
                                log_path=mpi_log_path)

        self.file_from_make = self.local_dir.joinpath('lib', 'libpetsc.a')

        self.extra_config_options = {'with-clanguage': 'cxx',
                                     'with-mpi': 1,
                                     'with-precision': 'double',
                                     'with-scalar-type': 'real',
                                     'with-shared-libraries': 0,
                                     'download-fblaslapack': 1,
                                     'download-f2cblaslapack': 1}

        if self.config.getboolean('required', 'mpi') or \
                not self.use_preinstalled:
            self.extra_config_options['with-mpi-dir'] = f'{self.local_dir}'

    @staticmethod
    def get_configure_command(config_options=None):
        """
        Get the command to configure the package.

        Notes
        -----
        Configuring happens through python 2
        https://github.com/petsc/petsc/blob/master/configure

        Parameters
        ----------
        config_options : dict
            Configuration options to use with `./configure`.
            The configuration options will be converted to `--key=val` during
            runtime

        Returns
        -------
        config_str : str
            The configuration command
        """

        options = ''
        if config_options is not None:
            for key, val in config_options.items():
                if val is not None:
                    options += f' --{key}={val}'
                else:
                    options += f' --{key}'

        config_str = f'python2 ./configure{options}'
        return config_str

    def get_petsc_arch(self):
        """
        Returns the os dependent PETSC_ARCH variable

        Returns
        -------
        petsc_arch : str
            The PETSC_ARCH variable
        """

        tar_dir = self.get_tar_dir(self.get_tar_file_path(self.petsc_url))
        petsc_arch = list(tar_dir.glob('arch*'))[0].name

        return petsc_arch

    def make(self, path):
        """
        Make the package using make all and make test

        Parameters
        ----------
        path : Path or str
            Path to the get_configure_command file
        """

        petsc_dir = f'PETSC_DIR={self.install_dir}/petsc-{self.petsc_version}'

        petsc_arch = f'PETSC_ARCH={self.get_petsc_arch()}'

        make_all_str = f'make {petsc_dir} {petsc_arch} all'
        self.run_subprocess(make_all_str, path)

        make_install_str = f'make {petsc_dir} {petsc_arch} install'
        self.run_subprocess(make_install_str, path)

        make_test_str = f'make PETSC_DIR={self.local_dir} PETSC_ARCH= test'
        self.run_subprocess(make_test_str, path)

    def install(self):
        """
        Installs PETSc and its dependencies
        """

        self.install_dependencies()

        self.logger.info('Installing PETSc')
        self.install_package(url=self.petsc_url,
                             file_from_make=self.file_from_make,
                             path_config_log='configure.log',
                             extra_config_option=self.extra_config_options,
                             overwrite_on_exist=self.overwrite_on_exist)
        self.logger.info('Installation completed successfully')

    def install_dependencies(self):
        """
        Install PETSc dependencies
        """

        self.mpi.install()



from pathlib import Path
from bout_install.InstallerUsingGit import InstallerUsingGit


class BOUTPPInstaller(InstallerUsingGit):
    """
    Installer object for installing BOUT++
    """

    def __init__(self,
                 config_path=Path(__file__).parent.joinpath('config.ini'),
                 log_path=Path(__file__).parents[1].joinpath('log',
                                                             'boutpp.log'),
                 overwrite_on_exist=False):
        """
        Gets the BOUT++ version, sets the BOUT++ url and calls the super
        constructor

        Notes
        -----
        The BOUTPPInstaller does not install its dependencies.
        See bout_install.main.install_bout for installation of dependencies

        Parameters
        ----------
        config_path : Path or str
            The path to the get_configure_command file
        log_path : None or Path or str
            Path to the log file containing the log of Installer.
            If None, the log will directed to stderr
        overwrite_on_exist : bool
            Whether to overwrite the package if it is already found
        """

        self.overwrite_on_exist = overwrite_on_exist
        section = 'bout_options'
        name = 'BOUT-dev'

        super().__init__(name,
                         section,
                         config_path=config_path,
                         log_path=log_path)

        self.boutpp_url = 'https://github.com/boutproject/BOUT-dev.git'
        self.file_from_make = self.git_dir.joinpath('lib', 'libbout++.a')

        checks = self.config['bout_options']['enable_checks']
        optimize = self.config['bout_options']['enable_optimize']

        self.extra_config_options = {'enable-checks': checks,
                                     'enable-optimize': optimize}

        if self.config.getboolean('required', 'fftw') or \
                not self.use_preinstalled:
            self.extra_config_options['with-fftw'] = self.local_dir
        if self.config.getboolean('required', 'netcdf') or \
                not self.use_preinstalled:
            self.extra_config_options['with-netcdf'] = self.local_dir

        if self.config.getboolean('required', 'hdf5'):
            self.extra_config_options['with-hdf5'] =\
                self.local_dir.joinpath('bin', 'h5cc')
        if self.config.getboolean('optional', 'sundials'):
            self.extra_config_options['with-sundials'] = self.local_dir
        if self.config.getboolean('optional', 'petsc'):
            self.extra_config_options['with-petsc'] = self.local_dir
        if self.config.getboolean('optional', 'slepc'):
            self.extra_config_options['with-slepc'] = self.local_dir

    def install(self):
        """
        Installs the BOUT++ package
        """

        self.logger.info('Installing BOUT++')
        self.install_package(url=self.boutpp_url,
                             file_from_make=self.file_from_make,
                             overwrite_on_exist=self.overwrite_on_exist,
                             extra_config_option=self.extra_config_options)
        self.logger.info('Installation completed successfully')



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import argparse
import configparser
from pathlib import Path
from bout_install.git_installer.BOUTPPInstaller import BOUTPPInstaller
from bout_install.installer.CMakeInstaller import CMakeInstaller
from bout_install.installer.FFMPEGInstaller import FFMPEGInstaller
from bout_install.installer.FFTWInstaller import FFTWInstaller
from bout_install.installer.GCCInstaller import GCCInstaller
from bout_install.installer.HDF5Installer import HDF5Installer
from bout_install.installer.MPIInstaller import MPIInstaller
from bout_install.installer.NetCDFInstaller import NetCDFInstaller
from bout_install.installer.PETScInstaller import PETScInstaller
from bout_install.installer.SLEPcInstaller import SLEPcInstaller
from bout_install.cmake_installer.SundialsInstaller import SundialsInstaller


def install_bout(config_path=None, add_to_bashrc=False):
    """
    Function which installs BOUT++ and its dependencies.

    Parameters
    ----------
    config_path : None or str or Path
        Path to the configuration file
        If None, the default configuration in bout_install.config.ini is
        used
    add_to_bashrc : bool
        Whether or not to add binaries and library path of the dependencies
        to .bashrc
    """

    if config_path is None:
        root_dir = Path(__file__).absolute().parents[1]
        config_path = root_dir.joinpath('bout_install', 'config.ini')

    # String to print when installation is complete
    final_str = '\n'

    config = configparser.ConfigParser(allow_no_value=True)
    with config_path.open() as f:
        config.read_file(f)

    if config.getboolean('optional', 'gcc'):
        print('Installing gcc...')
        gcc_installer = GCCInstaller(config_path=config_path)
        gcc_installer.install()
        final_str += (f'export PATH="'
                      f'{gcc_installer.local_dir.joinpath("bin")}:$PATH"\n')
        final_str += (f'export LD_LIBRARY_PATH = '
                      f'{gcc_installer.local_dir.joinpath("lib64")}:'
                      f'$LD_LIBRARY_PATH"\n')
        print('..done')

    if config.getboolean('required', 'mpi'):
        print('Installing mpi...')
        mpi_installer = MPIInstaller(config_path=config_path)
        mpi_installer.install()
        print('...done')

    if config.getboolean('optional', 'cmake'):
        print('Installing cmake...')
        cmake_installer = CMakeInstaller(config_path=config_path)
        cmake_installer.install()
        print('...done')

    if config.getboolean('optional', 'ffmpeg'):
        print('Installing ffmpeg...')
        ffmpeg_installer = FFMPEGInstaller(config_path=config_path)
        ffmpeg_installer.install()
        print('...done')

    if config.getboolean('required', 'fftw'):
        print('Installing fftw...')
        fftw_installer = FFTWInstaller(config_path=config_path)
        fftw_installer.install()
        print('...done')

    if config.getboolean('required', 'hdf5'):
        print('Installing hd5...')
        hdf5_installer = HDF5Installer(config_path=config_path)
        hdf5_installer.install()
        print('...done')

    if config.getboolean('required', 'netcdf'):
        print('Installing netcdf...')
        netcdf_installer = NetCDFInstaller(config_path=config_path)
        netcdf_installer.install()
        print('...done')

    if config.getboolean('optional', 'sundials'):
        print('Installing sundials...')
        sundials_installer = SundialsInstaller(config_path=config_path)
        sundials_installer.install()
        print('...done')

    if config.getboolean('optional', 'petsc'):
        print('Installing petsc...')
        petsc_installer = PETScInstaller(config_path=config_path)
        petsc_installer.install()
        print('...done')

    if config.getboolean('optional', 'slepc'):
        print('Installing slepc...')
        slepc_installer = SLEPcInstaller(config_path=config_path)
        slepc_installer.install()
        print('...done')

    print('Installing BOUT++...')
    boutpp_installer = BOUTPPInstaller(config_path=config_path)
    boutpp_installer.install()
    print('...done')

    final_str += (f'export LD_LIBRARY_PATH='
                  f'{boutpp_installer.local_dir.joinpath("lib")}:'
                  f'LD_LIBRARY_PATH\n\n')

    if add_to_bashrc:
        add_str_to_bashrc(final_str)
    else:
        print('Make sure that all binaries and libraries are in the PATH')
        print('You can do so by making sure that the following is in .bashrc:')
        print(final_str)


def add_str_to_bashrc(bashrc_str):
    """
    Adds the bashrc_str to .bashrc

    .bashrc will be created if non-existent

    Parameters
    ----------
    bashrc_str : str
        The string to add to bashrc
    """

    bashrc_str = f'# Added by bout_installer.py\n{bashrc_str}'
    bashrc_path = Path.home().joinpath('.bashrc')

    if not bashrc_path:
        bashrc_path.touch()

    print(f'Adding following to .bashrc:\n{bashrc_str}')

    with bashrc_path.open('a') as f:
        f.write(bashrc_str)


def get_args():
    """
    Returns the argument from the argument parser

    Returns
    -------
    config_path : None or str or Path
        Path to the configuration file
        If None, the default configuration in bout_install.config.ini is
        used
    add_to_bashrc : bool
        Whether or not to add binaries and library path of the dependencies
        to .bashrc
    """

    root_dir = Path(__file__).absolute().parents[1]
    config_path = root_dir.joinpath('bout_install', 'config.ini')

    parser = \
        argparse.ArgumentParser(description='Install BOUT++ with dependencies')

    parser.add_argument('-c',
                        '--config',
                        help=f'Path to the configuration file. '
                             f'Default is {config_path}',
                        default=str(config_path))
    parser.add_argument('-a',
                        '--add_to_bashrc',
                        help='If set, paths to binaries and libraries of '
                             'dependencies will be added to .bashrc. '
                             'Default is false',
                        action='store_true',
                        default=False)

    args = parser.parse_args()

    config_path = Path(args.config).absolute()
    add_to_bashrc = args.add_to_bashrc

    return config_path, add_to_bashrc


def bout_install_command_line():
    """
    The main routine which installs BOUT++ and dependencies

    Can be used for command line interface
    """
    config_path, add_to_bashrc = get_args()
    install_bout(config_path, add_to_bashrc=add_to_bashrc)



from pathlib import Path
from bout_install.InstallerUsingCMake import InstallerUsingCMake


class SundialsInstaller(InstallerUsingCMake):
    """
    Installer object for installing Sundials
    """

    def __init__(self,
                 config_path=Path(__file__).parent.joinpath('config.ini'),
                 log_path=Path(__file__).parents[1].joinpath('log',
                                                             'sundials.log'),
                 overwrite_on_exist=False):
        """
        Gets the Sundials version, sets the url and calls the super constructor

        Parameters
        ----------
        config_path : Path or str
            The path to the get_configure_command file
        log_path : None or Path or str
            Path to the log file containing the log of Installer.
            If None, the log will directed to stderr
            Whether to overwrite the package if it is already found
        """

        self.overwrite_on_exist = overwrite_on_exist

        super().__init__(config_path=config_path, log_path=log_path)

        self.sundials_version = self.config['versions']['sundials']
        self.sundials_url = (f'http://computation.llnl.gov/projects/'
                             f'sundials/download/'
                             f'sundials-{self.sundials_version}.tar.gz')

        self.file_from_make = self.local_dir.joinpath('lib',
                                                      'libsundials_cvode.a')

        self.extra_config_options = \
            {'DEXAMPLES_INSTALL_PATH': str(self.examples_dir),
             'DOPENMP_ENABLE': 'ON',
             'DMPI_ENABLE': 'ON',
             'DCMAKE_LINKER': f'{self.local_dir.joinpath("lib")}',
             'DBUILD_SHARED_LIBS': 'OFF'}

    def install(self):
        """
        Installs the Sundials package
        """

        self.logger.info('Installing Sundials')
        self.install_package(url=self.sundials_url,
                             file_from_make=self.file_from_make,
                             extra_cmake_option=self.extra_config_options,
                             overwrite_on_exist=self.overwrite_on_exist)
        self.logger.info('Installation completed successfully')



from pathlib import Path
from bout_install.Installer import Installer


class CMakeInstaller(Installer):
    """
    Installer object for installing CMake
    """

    def __init__(self,
                 config_path=Path(__file__).parent.joinpath('config.ini'),
                 log_path=Path(__file__).parents[1].joinpath('log',
                                                             'cmake.log'),
                 overwrite_on_exist=False):
        """
        Gets the CMake version, sets the CMake url, calls the super constructor

        Parameters
        ----------
        config_path : Path or str
            The path to the get_configure_command file
        log_path : None or Path or str
            Path to the log file containing the log of Installer.
            If None, the log will directed to stderr
        overwrite_on_exist : bool
            Whether to overwrite the package if it is already found
        """

        self.overwrite_on_exist = overwrite_on_exist

        super().__init__(config_path=config_path, log_path=log_path)

        self.cmake_version = self.config['versions']['cmake']
        cmake_major_minor_version = '.'.join(self.cmake_version.split('.')[:2])
        self.cmake_url = (f'http://cmake.org/files/'
                          f'v{cmake_major_minor_version}/'
                          f'cmake-{self.cmake_version}.tar.gz')

        self.file_from_make = self.local_dir.joinpath('bin', 'cmake')

    def run_configure(self,
                      tar_dir,
                      config_log_path,
                      extra_config_option,
                      overwrite_on_exist):
        """
        Configures the package with the bootstrap script

        Parameters
        ----------
        tar_dir : Path
            Directory of the tar file
        config_log_path : Path
            Path to config.log
        extra_config_option:
            Configure option to include.
            --prefix=self.local_dir is already added as an option
        overwrite_on_exist : bool
            Whether to overwrite the package if it is already found
        """

        # The bootstrap of CMake does not necessarily create a config.log
        # file, but it will at least create a Makefile which didn't exist before

        make_path = Path(config_log_path).parent.joinpath('Makefile')

        if not make_path.is_file() or overwrite_on_exist:
            config_options = dict(prefix=str(self.local_dir))
            if extra_config_option is not None:
                config_options = {**config_options, **extra_config_option}
            self.logger.info(f'Configuring with options {config_options}')
            config_str = \
                self.get_configure_command(config_options=config_options)

            # Calling the bootstrap script rather than configure
            config_str = config_str.replace('configure', 'bootstrap')
            self.run_subprocess(config_str, tar_dir)
        else:
            self.logger.info(f'{make_path} found, skipping configuring')

    def install(self):
        """
        Installs the CMake package
        """

        self.logger.info('Installing CMake')
        self.install_package(url=self.cmake_url,
                             file_from_make=self.file_from_make,
                             overwrite_on_exist=self.overwrite_on_exist)
        self.logger.info('Installation completed successfully')



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import re
import setuptools
from pathlib import Path

name = 'bout_install'
root_path = Path(__file__).parent
init_path = root_path.joinpath(name, '__init__.py')
readme_path = root_path.joinpath('README.md')

# https://packaging.python.org/guides/single-sourcing-package-version/
with init_path.open('r') as f:
    version_file = f.read()
    version_match = re.search(r"^__version__ = ['\"]([^'\"]*)['\"]",
                              version_file, re.M)
    if version_match:
        version = version_match.group(1)
    else:
        raise RuntimeError('Unable to find version string.')

with readme_path.open('r') as f:
    long_description = f.read()

setuptools.setup(
    name=name,
    version=version,
    author='Michael Løiten',
    author_email='michael.l.magnussen@gmail.com',
    description='Python package to install BOUT++ and its dependencies',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/CELMA-project/bout_install',
    packages=setuptools.find_packages(),
    keywords=['bout++', 'bout', 'installation', 'plasma', 'turbulence'],
    install_requires=['requests>=2.20.1'],
    package_data={
        # Include all .ini files
        '': ['*.ini']
    },
    classifiers=[
        'Programming Language :: Python :: 3',
        ('License :: OSI Approved :: '
         'GNU Lesser General Public License v3 or later (LGPLv3+)'),
        'Operating System :: OS Independent',
    ],
    entry_points={'console_scripts': [
        'bout_install = bout_install.main:bout_install_command_line'
    ]},
)



import configparser
import logging
import os
import requests
import shutil
import subprocess
import tarfile
from pathlib import Path


class Installer(object):
    """
    Class for building packages from scratch

    This class serves as the top-most parent class for most of the installer
    classes.

    Contains member functions for
    * Downloading a tar file for a package
    * Untar the downloaded tar file for the package
    * Configure the package
    * Making the package (including make install)

    Examples
    --------
    The following example will install FFTW.
    See also the examples of the child-classes of the Installer

    >>> from bout_install.Installer import Installer
    >>>
    >>> # Instantiate the installer
    >>> installer = Installer()
    >>>
    >>> # Get the fftw-version from the configuration
    >>> fftw_version = installer.config['versions']['fftw']
    >>> fftw_url = f'http://www.fftw.org/fftw-{fftw_version}.tar.gz'
    >>>
    >>> bin_file = installer.local_dir.joinpath('bin', 'fftw-wisdom')
    >>>
    >>> # Install FFTW
    >>> installer.install_package(fftw_url, bin_file)
    """

    def __init__(self,
                 config_path=Path(__file__).parent.joinpath('config.ini'),
                 log_path=None):
        """
        Makes the logger and installation paths (obtained from config.ini)

        Notes
        -----
        `self.local_dir.joinpath('bin')` will be set to the environments
        `PATH` variable and `self.local_dir.joinpath('lib')` will be set to the
        environments `LD_LIBRARY_PATH` variable in order to ensure a proper
        installation

        Parameters
        ----------
        config_path : Path or str
            The path to the get_configure_command file
        log_path : None or Path or str
            Path to the log file containing the log of Installer.
            If None, the log will directed to stderr
        """

        self.config = configparser.ConfigParser(allow_no_value=True)
        with Path(config_path).open() as f:
            self.config.read_file(f)

        # Set input
        self.log_path = log_path

        # Obtain install options
        self.use_preinstalled = \
            self.config.getboolean('install_options', 'use_preinstalled')
        main_dir = self.config['install_options']['main_dir']
        install_dir = self.config['install_options']['install_dir']
        local_dir = self.config['install_options']['local_dir']
        examples_dir = self.config['install_options']['examples_dir']

        self.main_dir = main_dir if main_dir != '' else None
        self.install_dir = install_dir if install_dir != '' else None
        self.local_dir = local_dir if local_dir != '' else None
        self.examples_dir = examples_dir if examples_dir != '' else None

        # Setup the install dirs
        self.setup_install_dirs(main_dir=self.main_dir,
                                install_dir=self.install_dir,
                                local_dir=self.local_dir,
                                examples_dir=self.examples_dir)

        # Set the environment variables
        # Set the local path first
        os.environ['PATH'] = (f'{self.local_dir.joinpath("bin")}'
                              f'{os.pathsep}'
                              f'{os.environ["PATH"]}')
        if 'LD_LIBRARY_PATH' not in os.environ:
            os.environ['LD_LIBRARY_PATH'] = f'{self.local_dir.joinpath("lib")}'
        else:
            os.environ['LD_LIBRARY_PATH'] = (f'{self.local_dir.joinpath("lib")}'
                                             f'{os.pathsep}'
                                             f'{os.environ["LD_LIBRARY_PATH"]}')

        # Declare other class variables
        self.config_log_path = None

        # Setup the logger
        self._setup_logger()

    def _setup_logger(self):
        """
        Sets up the logger instance.
        """
        formatter = logging.Formatter('[{asctime}][{levelname:<7}] {message}',
                                      style='{')

        if self.log_path is not None:
            log_path = Path(self.log_path).absolute()
            log_dir = log_path.parent
            log_dir.mkdir(exist_ok=True, parents=True)
            handler = logging.FileHandler(str(log_path))
        else:
            handler = logging.StreamHandler()

        handler.setFormatter(formatter)

        self.logger = logging.getLogger('bout_install')
        self.logger.setLevel(logging.INFO)
        self.logger.addHandler(handler)

    def setup_install_dirs(self,
                           main_dir=None,
                           install_dir=None,
                           local_dir=None,
                           examples_dir=None):
        """
        Set the install directories for the packages

        Parameters
        ----------
        main_dir : None or str or Path
            The super directory of install_dir, local_dir and example_dir
            (if not set).
            If None, the home directory will be used.
            install_dir, local_dir and example_dir have precedence over main_dir
        install_dir : None or str or Path
            The directory to put the files to install from.
             If None, the directory will be made under main_dir
        local_dir : None or str or Path
            The directory to put the installed files.
             If None, the directory will be made under main_dir
        examples_dir : None or str or Path
            The directory to put the examples (needed for installing ffmpeg).
            If None, the directory will be made under main_dir
        """

        if main_dir is None:
            self.main_dir = Path(__file__).home().absolute()
        else:
            self.main_dir = Path(main_dir).absolute()

        if install_dir is None:
            self.install_dir = self.main_dir.joinpath('install')
        else:
            self.install_dir = Path(install_dir).absolute()

        if local_dir is None:
            self.local_dir = self.main_dir.joinpath('local')
        else:
            self.local_dir = Path(local_dir).absolute()

        if examples_dir is None:
            self.examples_dir = self.main_dir.joinpath('examples')
        else:
            self.examples_dir = Path(examples_dir).absolute()

        # Make the directories
        if not (install_dir is None and
                local_dir is None and
                examples_dir is None):
            self.main_dir.mkdir(parents=True, exist_ok=True)

        self.install_dir.mkdir(parents=True, exist_ok=True)
        self.local_dir.mkdir(parents=True, exist_ok=True)
        self.examples_dir.mkdir(parents=True, exist_ok=True)

    def get_tar_file(self, url):
        """
        Obtain a tar file from url

        Parameters
        ----------
        url : str
            The url to get the tar file from
        """

        try:
            response = requests.get(url, stream=True)
        except requests.exceptions.SSLError:
            msg = (f'SSL error occurred in {url}, trying to download without '
                   f'SSL verification. Use with care!')
            self.logger.warning(msg)
            response = requests.get(url, stream=True, verify=False)

        response.raise_for_status()

        tar_file_path = self.get_tar_file_path(url)

        with tar_file_path.open('wb') as f:
            # Decode in case transport encoding was applied
            # https://stackoverflow.com/questions/32463419/having-trouble-getting-requests-2-7-0-to-automatically-decompress-gzip
            response.raw.decode_content = True
            shutil.copyfileobj(response.raw, f)

    def get_tar_file_path(self, url):
        """
        Returns the path to the tar file

        Parameters
        ----------
        url : str
            The url to get the tar file from

        Returns
        -------
        tar_file_path : Path
            The path to the tar file
        """

        # The file name is the last part of the url
        file_name = url.split('/')[-1]
        tar_file_path = self.install_dir.joinpath(file_name)
        return tar_file_path

    @staticmethod
    def untar(tar_path):
        """
        Untar a tar file

        Parameters
        ----------
        tar_path : str or Path
            Tar file to extract
        """

        tar_path = Path(tar_path).absolute()
        tar_extract_dir = tar_path.parent

        tar = tarfile.open(tar_path)
        tar.extractall(path=tar_extract_dir)
        tar.close()

    @staticmethod
    def get_tar_dir(tar_path):
        """
        Returns the path to the tar directory (directory of untarred files)

        Parameters
        ----------
        tar_path : str or Path
            Path to tar file

        Returns
        -------
        tar_dir : Path
            The untarred directory
        """

        dir_name = Path(tarfile.open(tar_path).getnames()[0]).parts[0]
        tar_dir = Path(tar_path).absolute().parent.joinpath(dir_name)
        return tar_dir

    @staticmethod
    def get_configure_command(config_options=None):
        """
        Get the command to configure the package

        Parameters
        ----------
        config_options : dict
            Configuration options to use with `./configure`.
            The configuration options will be converted to `--key=val` during
            runtime

        Returns
        -------
        config_str : str
            The configuration command
        """

        options = ''
        if config_options is not None:
            for key, val in config_options.items():
                if val is not None:
                    options += f' --{key}={val}'
                else:
                    options += f' --{key}'

        config_str = f'./configure{options}'
        return config_str

    def run_subprocess(self, command, path):
        """
        Run a subprocess

        Parameters
        ----------
        command : str
            The command to run
        path : Path or str
            Path to the location to run the command from
        """

        result = subprocess.run(command.split(),
                                stdout=subprocess.PIPE,
                                stderr=subprocess.PIPE,
                                cwd=path)

        if result.returncode != 0:
            self._raise_subprocess_error(result)

    def make(self, path):
        """
        Make the package

        Parameters
        ----------
        path : Path or str
            Path to the get_configure_command file
        """

        make_str = 'make'
        self.run_subprocess(make_str, path)

        make_install_str = 'make install'
        self.run_subprocess(make_install_str, path)

    def run_download_tar(self, url, tar_file_path, overwrite_on_exist):
        """
        Downloads the tar-file if not found

        Parameters
        ----------
        url : str
            The url to download the tar-file from
        tar_file_path : Path
            Path to the tar file
        overwrite_on_exist : bool
            Whether to overwrite the package if it is already found
        """

        if not tar_file_path.is_file() or overwrite_on_exist:
            self.logger.info(f'Downloading {url}')
            self.get_tar_file(url)
        else:
            self.logger.info(f'{tar_file_path} found, skipping download')

    def run_untar(self, tar_file_path, tar_dir, overwrite_on_exist):
        """
        Untars the tar file

        Parameters
        ----------
        tar_file_path : Path
            Path to the tar file
        tar_dir : Path
            Directory to the tar file
        overwrite_on_exist : bool
            Whether to overwrite the package if it is already found
        """

        if not tar_dir.is_dir() or overwrite_on_exist:
            self.logger.info(f'Untarring {tar_file_path}')
            self.untar(tar_file_path)
        else:
            self.logger.info(f'{tar_dir} found, skipping untarring')

    def run_configure(self,
                      tar_dir,
                      config_log_path,
                      extra_config_option,
                      overwrite_on_exist):
        """
        Configures the package

        Parameters
        ----------
        tar_dir : Path
            Directory of the tar file
        config_log_path : Path
            Path to the Makefile
        extra_config_option:
            Configure option to include.
            --prefix=self.local_dir is already added as an option
        overwrite_on_exist : bool
            Whether to overwrite the package if it is already found
        """

        if not config_log_path.is_file() or overwrite_on_exist:
            config_options = dict(prefix=str(self.local_dir))
            if extra_config_option is not None:
                config_options = {**config_options, **extra_config_option}

            config_str = \
                self.get_configure_command(config_options=config_options)

            self.logger.info(f'Configuring with: {config_str}')
            self.run_subprocess(config_str, tar_dir)
        else:
            self.logger.info(f'{config_log_path} found, skipping configuring')

    def run_make(self, tar_dir, file_from_make, overwrite_on_exist):
        """
        Runs make and make install

        Parameters
        ----------
        tar_dir : Path
            Directory of the tar file
        file_from_make : Path or str
            File originating from the make processes (used to check if the
            package has been made)
        overwrite_on_exist : bool
            Whether to overwrite the package if it is already found
        """

        if not file_from_make.is_file() or overwrite_on_exist:
            self.logger.info(f'Making (including make install)')
            self.make(tar_dir)
        else:
            self.logger.info(f'{file_from_make} found, skipping making')

    def install_package(self,
                        url,
                        file_from_make,
                        path_config_log='config.log',
                        overwrite_on_exist=False,
                        extra_config_option=None):
        """
        Installs a package if it's not installed

        Parameters
        ----------
        url : str
            Url to the tar file of the package
        file_from_make : Path or str
            File originating from the make processes (used to check if the
            package has been made)
        path_config_log : str or Path
            Name of the log file for configure relative to the configuration
            file
        overwrite_on_exist : bool
            Whether to overwrite the package if it is already found
        extra_config_option : dict
            Configure option to include.
            The installation prefix of self.local_dir is already added as an
            option
        """

        # Download the tar file
        tar_file_path = self.get_tar_file_path(url)
        self.run_download_tar(url, tar_file_path, overwrite_on_exist)

        # Untar
        tar_dir = self.get_tar_dir(tar_file_path)
        self.run_untar(tar_file_path, tar_dir, overwrite_on_exist)

        # Configure and make
        config_log_path = tar_dir.joinpath(path_config_log)
        self.run_configure(tar_dir,
                           config_log_path,
                           extra_config_option,
                           overwrite_on_exist)
        self.run_make(tar_dir, file_from_make, overwrite_on_exist)

    def _raise_subprocess_error(self, result):
        """
        Raises errors from the subprocess in a clean way

        Parameters
        ----------
        result : subprocess.CompletedProcess
            The result from the subprocess
        """

        self.logger.error('Subprocess failed with stdout:')
        self.logger.error(result.stdout)
        self.logger.error('and stderr:')
        self.logger.error(result.stderr)

        result.check_returncode()



import os
import shutil
from pathlib import Path
from bout_install.Installer import Installer
from bout_install.installer.HDF5Installer import HDF5Installer


class NetCDFInstaller(Installer):
    """
    Installer object for installing NetCDF
    """

    def __init__(self,
                 config_path=Path(__file__).parent.joinpath('config.ini'),
                 netcdf_log_path=
                 Path(__file__).parents[1].joinpath('log', 'netcdf.log'),
                 netcdf_cxx_log_path=
                 Path(__file__).parents[1].joinpath('log', 'netcdf_cxx.log'),
                 hdf5_log_path=
                 Path(__file__).parents[1].joinpath('log', 'hdf5.log'),
                 overwrite_on_exist=False):
        """
        Gets the version and url of NetCDF and calls the super constructor

        The constructor will also make an object of the HDF5 installer and
        the CXX interface installer

        Parameters
        ----------
        config_path : Path or str
            The path to the get_configure_command file
        netcdf_log_path : None or Path or str
            Path to the log file for NetCDF
            If None, the log will directed to stderr
        netcdf_cxx_log_path : None or Path or str
            Path to the log file for the NetCDF CXX interface
            If None, the log will directed to stderr
        hdf5_log_path : None or Path or str
            Path to the log file for HDF5
            If None, the log will directed to stderr
        overwrite_on_exist : bool
            Whether to overwrite the package if it is already found
        """

        self.overwrite_on_exist = overwrite_on_exist

        super().__init__(config_path=config_path, log_path=netcdf_log_path)

        self.netcdf_version = self.config['versions']['netcdf']
        self.netcdf_url = (f'http://www.unidata.ucar.edu/downloads/netcdf/ftp/'
                           f'netcdf-{self.netcdf_version}.tar.gz')

        # CPPFLAGS and LDFLAGS must be exported
        # https://www.unidata.ucar.edu/support/help/MailArchives/netcdf/msg13261.html
        # http://www.unidata.ucar.edu/software/netcdf/docs/getting_and_building_netcdf.html#build_default
        os.environ['CPPFLAGS'] = f'-I{self.local_dir.joinpath("include")}'
        os.environ['LDFLAGS'] = f'-L{self.local_dir.joinpath("lib")}'

        # Create dependency installer
        self.hdf5 = HDF5Installer(config_path=config_path,
                                  log_path=hdf5_log_path)

        # Create the cxx interface installer
        self.netcdf_cxx = NetCDFCXXInstaller(config_path=config_path,
                                             log_path=netcdf_cxx_log_path)

        self.file_from_make = self.local_dir.joinpath('bin', 'ncdump')

        self.extra_config_options = {'disable-dap': None}

    def install(self):
        """
        Installs HDF5, the NetCDF package and the CXX interface
        """

        self.install_dependencies()

        self.logger.info('Installing NetCDF')

        if shutil.which('ncdump') is None or not self.use_preinstalled:
            self.install_package(url=self.netcdf_url,
                                 file_from_make=self.file_from_make,
                                 extra_config_option=self.extra_config_options,
                                 overwrite_on_exist=self.overwrite_on_exist)
            self.logger.info('Installation completed successfully')
        else:
            self.logger.info('Found ncdump in PATH, skipping...')

        # Install the cxx interface
        self.netcdf_cxx.install()

    def install_dependencies(self):
        """
        Installs NetCDF dependencies
        """

        self.hdf5.install()


class NetCDFCXXInstaller(Installer):
    """
    Installer object for installing NetCDFs CXX interface
    """

    def __init__(self,
                 config_path=Path(__file__).parent.joinpath('config.ini'),
                 log_path=
                 Path(__file__).parents[1].joinpath('log', 'netcdf_cxx.log'),
                 overwrite_on_exist=False):
        """
        Gets the NetCDF CXX version, sets the NetCDF CXX url and calls the
        super constructor

        Parameters
        ----------
        config_path : Path or str
            The path to the get_configure_command file
        log_path : None or Path or str
            Path to the log file containing the log of Installer.
            If None, the log will directed to stderr
        overwrite_on_exist : bool
            Whether to overwrite the package if it is already found
        """

        self.overwrite_on_exist = overwrite_on_exist

        super().__init__(config_path=config_path, log_path=log_path)

        self.netcdf_cxx_version = self.config['versions']['netcdf_cxx']
        self.netcdf_cxx_url = (f'http://github.com/Unidata/netcdf-cxx4/archive/'
                               f'v{self.netcdf_cxx_version}.tar.gz')

        self.file_from_make = self.local_dir.joinpath('bin', 'ncxx4-config')

    def install(self):
        """
        Installs the NetCDF CXX interface
        """

        self.logger.info('Installing NetCDF CXX interface')

        if shutil.which('ncxx4-config') is None or not self.use_preinstalled:
            self.install_package(url=self.netcdf_cxx_url,
                                 file_from_make=self.file_from_make,
                                 overwrite_on_exist=self.overwrite_on_exist)
            self.logger.info('Installation completed successfully')
        else:
            self.logger.info('Found ncxx4-config in PATH, skipping...')
